package com.example.biblio;

import java.sql.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

public class Oeuvre {
    private int id;
    private String titre;
    private int anneePremiereParution;
    private List<Auteur> auteurs = new ArrayList<>();
    private List<Integer> auteurIds;
    private List<String> keywords; // Liste de mots-clés

    public Oeuvre() {
        this.keywords = new ArrayList<>();
    }

    public Oeuvre(String titre, int anneePremiereParution, List<Auteur> auteurs, List<String> keywords) {
        this.id = id;
        this.titre = titre;
        this.anneePremiereParution = anneePremiereParution;
        this.auteurs = auteurs;
        this.keywords = new ArrayList<>(keywords);

        if (this.keywords.isEmpty() || this.keywords.get(0) == null || this.keywords.get(0).isEmpty()) {
            throw new IllegalArgumentException("Le premier mot-clé est requis.");
        }

        if (this.keywords.size() > 5) {
            throw new IllegalArgumentException("Un maximum de 5 mots-clés est autorisé.");
        }
    }

    public static Oeuvre getOeuvrebyID(int oeuvreId) {
        String query = "SELECT * FROM Oeuvre WHERE id = " + oeuvreId;

        DatabaseConnection databaseConnection = new DatabaseConnection();
        Oeuvre oeuvre = null;
        try (Connection connection = databaseConnection.connect();
             Statement statement = connection.createStatement();
             ResultSet resultSet = statement.executeQuery(query)) {

            if (resultSet.next()) {
                oeuvre = new Oeuvre();
                oeuvre.setId(resultSet.getInt("id"));
                oeuvre.setTitre(resultSet.getString("titre"));
                oeuvre.setAnneePremiereParution(resultSet.getInt("anneePremiereParution"));

                // Récupérer les IDs des auteurs sous forme de chaîne
                String auteurIdsString = resultSet.getString("auteurs_ids");
                oeuvre.setAuteurIdsFromString(auteurIdsString); // Stockage des IDs sous forme de liste dans l'objet Oeuvre

                // Récupérer et définir les objets Auteur associés à ces IDs
                List<Auteur> auteurs = getAuteursByIds(auteurIdsString, connection);
                oeuvre.setAuteurs(auteurs);

                // Charger les mots-clés en tant que liste
                String keywords = resultSet.getString("mots_cles");
                if (keywords != null) {
                    oeuvre.setKeywords(Arrays.asList(keywords.split(",")));
                }
            }
        } catch (SQLException e) {
            System.out.println(e.getMessage());
            throw new RuntimeException(e);
        }

        return oeuvre;
    }

    public void setAuteurIdsFromString(String auteurIdsString) {
        if (auteurIdsString != null && !auteurIdsString.isEmpty()) {
            String[] idsArray = auteurIdsString.split(",");
            List<Integer> auteurIds = new ArrayList<>();

            for (String idStr : idsArray) {
                try {
                    auteurIds.add(Integer.parseInt(idStr.trim())); // Convertit chaque ID en entier après avoir supprimé les espaces
                } catch (NumberFormatException e) {
                    System.err.println("Erreur lors de la conversion d'un ID d'auteur : " + idStr);
                }
            }

            this.auteurIds = auteurIds; // Assigne la liste d'IDs au champ auteurIds de l'objet Oeuvre
        } else {
            this.auteurIds = new ArrayList<>(); // Initialise une liste vide si auteurIdsString est vide ou nul
        }
    }

    private static List<Auteur> getAuteursByIds(String auteurIdsString, Connection connection) throws SQLException {
        List<Auteur> auteurs = new ArrayList<>();
        if (auteurIdsString == null || auteurIdsString.isEmpty()) {
            return auteurs;
        }

        String[] auteurIds = auteurIdsString.split(",");
        String placeholders = String.join(",", Collections.nCopies(auteurIds.length, "?"));
        String query = "SELECT * FROM Auteur WHERE id IN (" + placeholders + ")";

        try (PreparedStatement preparedStatement = connection.prepareStatement(query)) {
            for (int i = 0; i < auteurIds.length; i++) {
                preparedStatement.setInt(i + 1, Integer.parseInt(auteurIds[i].trim()));
            }

            try (ResultSet resultSet = preparedStatement.executeQuery()) {
                while (resultSet.next()) {
                    Auteur auteur = new Auteur();
                    auteur.setId(resultSet.getInt("id"));
                    auteur.setPrenom(resultSet.getString("prenom"));
                    auteur.setNom(resultSet.getString("nom"));
                    auteur.setAnneeNaissance(resultSet.getInt("anneeNaissance"));
                    auteurs.add(auteur);
                }
            }
        }
        return auteurs;
    }

    @Override
    public String toString() {
        return titre;  // Afficher le titre de l'oeuvre dans la ComboBox
    }

    // Méthode pour définir les mots-clés avec une validation de taille et de contenu
    public void setKeywords(List<String> keywords) {
        if (keywords.isEmpty() || keywords.get(0) == null || keywords.get(0).isEmpty()) {
            throw new IllegalArgumentException("Le premier mot-clé est requis.");
        }

        if (keywords.size() > 5) {
            throw new IllegalArgumentException("Un maximum de 5 mots-clés est autorisé.");
        }

        this.keywords = new ArrayList<>(keywords);
    }

    public List<String> getKeywords() {
        return keywords;
    }

    // Getters et Setters
    public int getId() {
        return id;
    }

    public void setId(int id) {
        this.id = id;
    }

    public void setAuteurIds(List<Integer> auteurIds) {
        this.auteurIds = auteurIds;
    }

    public List<Integer> getAuteurIds() {
        return auteurIds;
    }

    public void setTitre(String titre) {
        this.titre = titre;
    }

    public void setAnneePremiereParution(int anneePremiereParution) {
        this.anneePremiereParution = anneePremiereParution;
    }

    public int getAnneePremiereParution() {
        return anneePremiereParution;
    }

    public String getTitre() {
        return titre;
    }

    // Méthode pour obtenir les IDs des auteurs sous forme de chaîne séparée par des virgules
    public String getAuteursIds() {
        return auteurs.stream()
                .map(auteur -> String.valueOf(auteur.getId()))
                .collect(Collectors.joining(","));
    }

    // Méthode pour définir la liste des auteurs
    public void setAuteurs(List<Auteur> auteurs) {
        this.auteurs = auteurs;
    }

    // Getter pour les noms des auteurs sous forme de chaîne
    public String getNomsAuteurs() {
        if (auteurs == null || auteurs.isEmpty()) {
            System.out.println("La liste 'auteurs' est vide ou n'a pas été initialisée pour l'œuvre : " + this.titre);
            return "Aucun auteur";  // Message de retour si la liste est vide
        }

        return auteurs.stream()
                .map(auteur -> auteur.getPrenom() + " " + auteur.getNom())
                .collect(Collectors.joining(", "));
    }
}