package com.example.biblio;

import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.time.LocalDate;

public class ReservationService {
    private final DatabaseConnection databaseConnection = new DatabaseConnection();

    public ReservationService() {
    }

    public void creerReservation(Usager usager, Exemplaire exemplaire) {
        System.out.println("Demande de réservation reçue");
        Connection connection = null;
        try {
            connection = databaseConnection.connect();

            // Vérifier si l'exemplaire est déjà emprunté
            if (exemplaireEstEmprunte(connection, exemplaire)) {
                throw new IllegalStateException("L'exemplaire est déjà emprunté.");
            }

            // Vérifier si l'usager peut encore emprunter des exemplaires
            if (!usagerPeutEmprunter(connection, usager)) {
                throw new IllegalStateException("L'usager a atteint le nombre maximal d'emprunts.");
            }

            int duree = tempsMaxEmprunt(usager, connection);

            // Créer l'emprunt
            try (PreparedStatement stmt = connection.prepareStatement(
                    "INSERT INTO Emprunt (dateEmprunt, dateRetour, usager_id, exemplaire_id, rendu) VALUES (?, ?, ?, ?, ?)")) {
                stmt.setDate(1, Date.valueOf(LocalDate.now()));
                stmt.setDate(2, Date.valueOf(LocalDate.now().plusDays(duree)));
                stmt.setLong(3, usager.getId());
                stmt.setLong(4, exemplaire.getId());
                stmt.setBoolean(5, false);
                stmt.executeUpdate();
            }

            // Mettre à jour l'exemplaire pour indiquer qu'il est emprunté
            try (PreparedStatement stmt = connection.prepareStatement(
                    "UPDATE Exemplaire SET emprunte = true WHERE id = ?")) {
                stmt.setLong(1, exemplaire.getId());
                stmt.executeUpdate();
            }

            System.out.println("Réservation effectuée avec succès.");

        } catch (SQLException e) {
            System.err.println("Erreur lors de la création de la réservation : " + e.getMessage());
        } finally {
            // Assurer la fermeture de la connexion après utilisation
            databaseConnection.disconnect();
        }
    }

    public boolean isExemplaireBorrowedByUser(Exemplaire exemplaire, Usager usager) {
        // Si l'exemplaire n'est pas emprunté, inutile de continuer
        if (!exemplaire.getEmprunte()) {
            return false;
        }

        String query = "SELECT COUNT(*) FROM Emprunt WHERE exemplaire_id = ? AND usager_id = ?";

        try (Connection connection = databaseConnection.connect();
             PreparedStatement preparedStatement = connection.prepareStatement(query)) {
            preparedStatement.setInt(1, exemplaire.getId());  // ID de l'exemplaire
            preparedStatement.setInt(2, usager.getId());  // ID de l'usager
            ResultSet resultSet = preparedStatement.executeQuery();

            if (resultSet.next()) {
                // Si l'utilisateur a un emprunt en cours pour cet exemplaire
                return resultSet.getInt(1) > 0;
            }
        } catch (SQLException e) {
            System.err.println("Erreur lors de la vérification de l'emprunt : " + e.getMessage());
        }

        return false;
    }

    private boolean exemplaireEstEmprunte(Connection connection, Exemplaire exemplaire) throws SQLException {
        try (PreparedStatement stmt = connection.prepareStatement(
                "SELECT emprunte FROM Exemplaire WHERE id = ?")) {
            stmt.setLong(1, exemplaire.getId());
            try (ResultSet rs = stmt.executeQuery()) {
                if (rs.next()) {
                    return rs.getBoolean("emprunte");
                }
            }
        }
        return false;
    }

    private boolean usagerPeutEmprunter(Connection connection, Usager usager) throws SQLException {
        int maxEmprunts = getNombreMaxEmprunts(connection, usager);
        int nombreEmpruntsActuels = getNombreEmpruntsEnCours(connection, usager);

        return nombreEmpruntsActuels < maxEmprunts;
    }

    private int tempsMaxEmprunt(Usager usager, Connection connection) throws SQLException {
        int id = usager.getCategorieUsagerId();
        // Créer l'emprunt
        String query = "SELECT dureeMaxEmprunt FROM CategorieUsager WHERE id = ?";

        try (PreparedStatement preparedStatement = connection.prepareStatement(query)) {
            preparedStatement.setInt(1, id);  // ID
            ResultSet resultSet = preparedStatement.executeQuery();

            if (resultSet.next()) {
                // Si l'utilisateur a un emprunt en cours pour cet exemplaire
                return resultSet.getInt("dureeMaxEmprunt");
            }
        } catch (SQLException e) {
            System.err.println("Erreur lors de la vérification de l'emprunt : " + e.getMessage());
        }
        return 0;
    }

    private int getNombreMaxEmprunts(Connection connection, Usager usager) throws SQLException {
        try (PreparedStatement stmt = connection.prepareStatement(
                "SELECT nombreMaxEmprunts FROM CategorieUsager WHERE id = ?")) {
            stmt.setLong(1, usager.getCategorieUsagerId());
            try (ResultSet rs = stmt.executeQuery()) {
                if (rs.next()) {
                    return rs.getInt("nombreMaxEmprunts");
                }
            }
        }
        throw new SQLException("Impossible de récupérer la catégorie de l'usager.");
    }

    private int getNombreEmpruntsEnCours(Connection connection, Usager usager) throws SQLException {
        try (PreparedStatement stmt = connection.prepareStatement(
                "SELECT COUNT(*) FROM Emprunt WHERE usager_id = ? AND rendu = false")) {
            stmt.setLong(1, usager.getId());
            try (ResultSet rs = stmt.executeQuery()) {
                if (rs.next()) {
                    return rs.getInt(1);
                }
            }
        }
        throw new SQLException("Impossible de récupérer le nombre d'emprunts actuels.");
    }

    public void annulerReservation(Usager usager, Exemplaire exemplaire) {
        try (Connection connection = databaseConnection.connect()) {

            // Mettre à jour l'exemplaire pour indiquer qu'il n'est plus emprunté
            try (PreparedStatement stmtExemplaire = connection.prepareStatement(
                    "UPDATE Exemplaire SET emprunte = false WHERE id = ?")) {
                stmtExemplaire.setLong(1, exemplaire.getId());
                stmtExemplaire.executeUpdate();
            }

            // Trouver l'emprunt correspondant à cet exemplaire et à cet usager, puis marquer comme "rendu"
            try (PreparedStatement stmtEmprunt = connection.prepareStatement(
                    "UPDATE Emprunt SET rendu = true WHERE exemplaire_id = ? AND usager_id = ? AND rendu = false")) {
                stmtEmprunt.setLong(1, exemplaire.getId());
                stmtEmprunt.setLong(2, usager.getId());
                stmtEmprunt.executeUpdate();
            }

        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
    }

    public Date dateMaxEmprunt(Exemplaire exemplaire) {
        try (Connection connection = databaseConnection.connect()) {

            // Trouver l'emprunt correspondant à cet exemplaire et à cet usager, puis marquer comme "rendu"
            try (PreparedStatement stmtEmprunt = connection.prepareStatement(
                    "SELECT dateRetour FROM Emprunt WHERE exemplaire_id = ? AND rendu = false")) {
                stmtEmprunt.setLong(1, exemplaire.getId());
                try (ResultSet rs = stmtEmprunt.executeQuery()) {
                    if (rs.next()) {
                        return rs.getDate("dateRetour");
                    }
                }
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        return null;
    }
}